<?php
//

class Billiondigital_Core_Helper_Filesystem extends Mage_Core_Helper_Abstract
{
    /**
     * @param $dir
     * @param bool $deleteRootToo
     */
    public function remove($dir, $deleteRootToo = false)
    {
        if (!is_dir($dir)) {
            @unlink($dir);
            return;
        }
        if (!$dh = @opendir($dir)) {
            return;
        }
        while (false !== ($obj = readdir($dh))) {
            if ($obj == '.' || $obj == '..') {
                continue;
            }

            if (!@unlink($dir . '/' . $obj)) {
                $this->remove($dir . '/' . $obj, true);
            }
        }

        closedir($dh);

        if ($deleteRootToo) {
            @rmdir($dir);
        }
        return;
    }

    /**
     * @param $src
     * @param $dst
     */
    public function copy($src, $dst)
    {
        if (is_file($src)) {
            $dir = dirname($dst);
            if (!is_dir($dir)) {
                mkdir($dir, 0777, true);
            }
            if (is_writable($dir)) {
                copy($src, $dst);
            } else {
                throw new Billiondigital_Core_PermissionException('Write permission denied: ' . $dst);
            }
        } else if (is_dir($src)) {
            $dir = opendir($src);
            @mkdir($dst, 0777, true);
            while (false !== ($file = readdir($dir))) {
                if (($file != '.') && ($file != '..')) {
                    if (is_dir($src . DS . $file)) {
                        $this->copy($src . DS . $file, $dst . DS . $file);
                    } else {
                        if (!is_writable($dst)) {
                            throw new Billiondigital_Core_PermissionException('Write permission denied: ' . $dst);
                        } else if (file_exists($dst . DS . $file) && !is_writable($dst . DS . $file)) {
                            throw new Billiondigital_Core_PermissionException('Write permission denied: ' . $dst . DS . $file);
                        } else {
                            copy($src . DS . $file, $dst . DS . $file);
                        }
                    }
                }
            }
            closedir($dir);
        } else {
            throw new Mage_Core_Exception('Fso copy: Wrong arguments');
        }
    }

    public function rename($old, $new, $safe = true)
    {
        if (!file_exists($old)) {
            throw new Billiondigital_Core_PermissionException('Fso rename: could not find source');
        } else if ($safe && file_exists($new)) {
            throw new Billiondigital_Core_PermissionException('Fso rename: target already exists');
        } else {
            $result = rename($old, $new);
            if (!$result) {
                throw new Billiondigital_Core_PermissionException('Fso rename: permission denied');
            }
        }
    }

    /**
     * @param $path string
     * @return array File list
     */
    public function enumerate($path, $deep = true, $re = null, $dirs = false)
    {
        $files = array();
        if (is_dir($path)) {
            if ($handle = opendir($path)) {
                while (($name = readdir($handle)) !== false) {
                    if (!preg_match('#^\.#', $name)) {
                        if (is_dir($path . '/' . $name)) {
                            if ($dirs) $files[] = $path . '/' . $name;
                            if (!$deep) continue;
                            $files = array_merge($files, $this->enumerate($path . '/' . $name, $deep, $re, $dirs));
                        } else {
                            if ($re && !preg_match($re, $name)) continue;
                            $files[] = $path . '/' . $name;
                        }
                    }
                }

                closedir($handle);
            }
        }

        return $files;
    }

    /**
     * @param $path
     * @param $contents
     */
    public function write($path, $contents, $flags = 0)
    {
        $dir = dirname($path);
        if (!file_exists($dir)) {
            mkdir($dir, 0777, true);
        }
        if (!is_writable($dir)) {
            throw new Billiondigital_Core_PermissionException('Write permission denied: ' . $dir);
        } else if (file_exists($path) && !is_writable($path)) {
            throw new Billiondigital_Core_PermissionException('Write permission denied: ' . $path);
        } else {
            file_put_contents($path, $contents, $flags);
        }
    }

    public function read($path)
    {
        $content = null;
        if (file_exists($path)) {
            $content = file_get_contents($path);
            if ($content === false)
                throw new Billiondigital_Core_PermissionException('Read permission denied: ' . $path);
        }
        return $content;
    }

    public function checkDirPermissions($path)
    {
        return is_dir($path) && is_writable($path) && is_readable($path);
    }

    public function checkFilePermissions($path)
    {
        return file_exists($path) && is_writable($path) && is_readable($path);
    }
}

//